package me.unurled.sacredrealms.cutscenes;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map.Entry;
import me.unurled.sacredrealms.cutscenes.commands.BlockCommand;
import me.unurled.sacredrealms.cutscenes.commands.CutsceneCommand;
import me.unurled.sacredrealms.cutscenes.commands.NPCCommand;
import me.unurled.sacredrealms.cutscenes.parse.CutsceneParser;
import me.unurled.sacredrealms.cutscenes.parse.MarkerParser;
import net.kyori.adventure.text.Component;
import net.kyori.adventure.text.format.NamedTextColor;
import org.bukkit.Location;
import org.bukkit.configuration.file.FileConfiguration;
import org.bukkit.configuration.file.YamlConfiguration;
import org.bukkit.entity.Player;
import org.bukkit.plugin.java.JavaPlugin;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public final class Cutscenes extends JavaPlugin {

    private static Cutscenes instance;

    private ArrayList<Cutscene> cutscenes;
    private HashMap<Cutscene, Player> currentPlayingCutscenes;
    private HashMap<Player, Location> originalLocation;
    private ArrayList<String> cutscenesId;

    private static Logger LOGGER = LoggerFactory.getLogger("Cutscenes");

    @Override
    public void onEnable() {
        // Plugin startup logic
        instance = this;

        saveDefaultConfig();

        commands();

        load();
    }

    void commands() {
        CutsceneCommand cutsceneCommand = new CutsceneCommand();
        getCommand("cutscenes").setExecutor(cutsceneCommand);
        getCommand("cutscenes").setTabCompleter(cutsceneCommand);

        NPCCommand npcCommand = new NPCCommand();
        getCommand("csnpc").setExecutor(npcCommand);
        getCommand("csnpc").setTabCompleter(npcCommand);

        BlockCommand blockCommand = new BlockCommand();
        getCommand("csblock").setExecutor(blockCommand);
        getCommand("csblock").setTabCompleter(blockCommand);
    }

    public void load() {
        cutscenes = new ArrayList<>();
        currentPlayingCutscenes = new HashMap<>();
        originalLocation = new HashMap<>();
        cutscenesId = new ArrayList<>();

        detectAndParseAllCutscenes();
    }

    @Override
    public void onDisable() {
        // Plugin shutdown logic
    }

    void detectAndParseAllCutscenes() {
        new File("plugins/SacredRealms").mkdir();
        File folder = new File("plugins/SacredRealms", "cutscenes");
        if (!folder.exists()) {
            folder.mkdir();
        } else {
            if (folder.isDirectory()) {
                Marker.setMarkers(new ArrayList<>());
                File[] files = folder.listFiles();
                if (files != null) {
                    for (File file : files) {
                        if (file.isDirectory()) { // cutscene id folder
                            File[] files1 = file.listFiles();
                            if (files1 != null) {
                                for (File file1 : files1) {
                                    if (file1.getName().equals("cutscene.yml")) {
                                        try {
                                            FileConfiguration config = YamlConfiguration.loadConfiguration(file1);
                                            Cutscene cutscene = CutsceneParser.parse(config);
                                            if (cutscene == null)
                                                continue;
                                            addCutscene(cutscene);
                                        } catch (Exception e) {
                                            LOGGER.error("Failed to parse cutscene " + file.getName() + "!");
                                        }
                                    } else if (file1.isDirectory()) {
                                        File[] files2 = file1.listFiles();
                                        if (files2 != null) {
                                            for (File file2 : files2) {
                                                if (file2.getName().equals("marker.yml")) {
                                                    try {
                                                        FileConfiguration config = YamlConfiguration.loadConfiguration(file2);
                                                        Marker marker = MarkerParser.parse(config);
                                                        if (marker == null)
                                                            continue;
                                                        Marker.getMarkers().add(marker);
                                                    } catch (Exception e) {
                                                        LOGGER.error("Failed to parse marker " + file.getName() + "!");
                                                    }
                                                }
                                            }
                                        }
                                    }
                                }
                            }
                        }
                    }
                }
            } else {
                LOGGER.error("The cutscenes folder is not a directory!");
                try {
                    folder.delete();
                    folder.mkdir();
                } catch (Exception e) {
                    LOGGER.error("Failed to delete the cutscenes folder!");
                }
            }
        }
        for (Cutscene cutscene : cutscenes) {
            cutscene.setMarkers();
            cutscene.calculateMarkers();
        }
    }

    public static Cutscenes getInstance() {
        return instance;
    }

    public ArrayList<Cutscene> getCutscenes() {
        return cutscenes;
    }

    public HashMap<Cutscene, Player> getCurrentPlayingCutscenes() {
        return currentPlayingCutscenes;
    }

    // TODO: make it save the changes
    public void createCutscene(String id) {
        Cutscene cutscene = new Cutscene(id, id);
        addCutscene(cutscene);
    }

    // TODO: make it save the changes
    public void removeCutscene(String id) {
        Cutscene cutscene = getCutscene(id);
        removeCutscene(cutscene);
    }

    public void addCutscene(Cutscene cutscene) {
        LOGGER.info("Adding cutscene " + cutscene.getID().toUpperCase() + " to the list of cutscenes!");
        cutscenes.add(cutscene);
        cutscenesId.add(cutscene.getID().toUpperCase());
    }

    public void removeCutscene(Cutscene cutscene) {
        cutscenes.remove(cutscene);
        cutscenesId.remove(cutscene.getID().toUpperCase());
    }

    public void addCurrentPlayingCutscene(Cutscene cutscene, Player p) {
        currentPlayingCutscenes.put(cutscene, p);
    }

    public void removeCurrentPlayingCutscene(Cutscene cutscene, Player p) {
        currentPlayingCutscenes.remove(cutscene, p);
    }

    public ArrayList<String> getCutscenesId() {
        return cutscenesId;
    }

    public void stopCutscene(Player p) {
        for (Cutscene cutscene : currentPlayingCutscenes.keySet()) {
            if (currentPlayingCutscenes.get(cutscene).equals(p)) {
                currentPlayingCutscenes.remove(cutscene, p);
                cutscene.stop(p);
            }
        }
    }

    public void pauseCutscenes(Player p) {
        for (Cutscene cutscene : currentPlayingCutscenes.keySet()) {
            if (currentPlayingCutscenes.get(cutscene).equals(p)) {
                cutscene.stop(p);
            }
        }
    }

    public void stopCutscenesForAllPlayers() {
        for (Cutscene cutscene : currentPlayingCutscenes.keySet()) {
            cutscene.stop(currentPlayingCutscenes.get(cutscene));
            currentPlayingCutscenes.remove(cutscene);
        }
    }

    public void playCutscene(Cutscene cutscene, Player p) {
        if (cutscene == null) {
            p.sendMessage(
                Component.text("There was an error with the cutscene!", NamedTextColor.RED));
            return;
        }
        if (cutscenes.contains(cutscene)) {
            if (!currentPlayingCutscenes.containsValue(p)) {
                currentPlayingCutscenes.put(cutscene, p);
                cutscene.play(p);
                LOGGER.info("Playing cutscene " + cutscene.getID() + " to player " + p.getName() + "!");
            }
        }
    }

    public Cutscene getCutscene(Player p){
        for (Entry<Cutscene, Player> entry : currentPlayingCutscenes.entrySet()) {
            if (entry.getValue().equals(p))
                return entry.getKey();
        }
        return null;
    }

    public Cutscene getCutscene(String id){
        if (!cutscenesId.contains(id.toUpperCase())) {
            return null;
        }
        for (Cutscene cutscene : cutscenes) {
            if (cutscene.getID().equalsIgnoreCase(id.toUpperCase()))
                return cutscene;
        }
        return null;
    }

    public void setOriginalLocation(Player p, Location loc){
        originalLocation.put(p, loc);
    }

    public Location getOriginalLocation(Player p){
        return originalLocation.get(p);
    }

    public static Logger getLOGGER() {
        return LOGGER;
    }
}
