package me.unurled.sacredrealms.cutscenes.parse;

import me.unurled.sacredrealms.cutscenes.Cutscene;
import me.unurled.sacredrealms.cutscenes.Cutscenes;
import org.bukkit.Bukkit;
import org.bukkit.Location;
import org.bukkit.configuration.file.FileConfiguration;

/**
 * File structure organisation :
 * ```plugins/SacredRealms/cutscenes/<cutscene_id>/cutscene.yml```
 * ```plugins/SacredRealms/cutscenes/<cutscene_id>/<marker_id>/marker.yml```

 * ```plugins/SacredRealms/cutscenes/<cutscene_id>/cutscene.yml``` contains the cutscene's name, ID, start and end location, and the list of markers IDs;
 * ```
 * name: "Cutscene name"
 * id: "CUTSCENE_ID" // must be unique and uppercase
 * start: "world,x,y,z,yaw,pitch"
 * end: "world,x,y,z,yaw,pitch"
 * markers:
 *  - "marker_id"
 *  - "marker_id"
 *  ...
 * ```
 */
public class CutsceneParser {

    public static Cutscene parse(FileConfiguration file) {
        String name = null;
        String id = null;
        Cutscene cutscene = null;
        if (file.contains("name")) { // name: "Cutscene name"
            name = file.getString("name");
        }
        if (file.contains("id")) { // id: "CUTSCENE_ID"
            id = file.getString("id").toUpperCase();
        }
        if (name != null && id != null && name.length() != 0 && id.length() != 0) {
            cutscene = new Cutscene(id.toUpperCase(), name);
        } else {
            Cutscenes.getLOGGER().error(
                "Error while parsing cutscene name or ID " + (name == null ? "(name)" : name + " ")
                    + (id == null ? "(id)" : id));
            return null;
        }
        double x = 0, y = 0, z = 0;
        float yaw = 0, pitch = 0;
        if (file.contains("start")) { // start: "world,x,y,z,yaw,pitch"
            try {
                String[] start = file.getString("start").split(",");
                x = Double.parseDouble(start[1]);
                y = Double.parseDouble(start[2]);
                z = Double.parseDouble(start[3]);
                yaw = Float.parseFloat(start[4]);
                pitch = Float.parseFloat(start[5]);
                Location location = new Location(Bukkit.getWorld(start[0]), x, y, z, yaw, pitch);
            } catch (Exception e) {
                Cutscenes.getLOGGER()
                    .error("Error while parsing cutscene start location: " + e.getMessage());
                return null;
            }
        }
        if (file.contains("end")) { // end: "world,x,y,z,yaw,pitch"
            try {
                String[] end = file.getString("end").split(",");
                x = Double.parseDouble(end[1]);
                y = Double.parseDouble(end[2]);
                z = Double.parseDouble(end[3]);
                yaw = Float.parseFloat(end[4]);
                pitch = Float.parseFloat(end[5]);
                Location location = new Location(Bukkit.getWorld(end[0]), x, y, z, yaw, pitch);
            } catch (Exception e) {
                Cutscenes.getLOGGER()
                    .error("Error while parsing cutscene end location: " + e.getMessage());
                return null;
            }
        }
        if (file.contains("markers")) { // markers: ["marker_id", "marker_id", ...]
            for (String marker : file.getStringList("markers")) {
                try {
                    cutscene.addMarkerId(Integer.parseInt(marker));
                } catch (Exception e) {
                    Cutscenes.getLOGGER().error(
                        "Error while parsing cutscene " + name + "/" + id + " marker ID: "
                            + e.getMessage());
                    return null;
                }
            }
        }
        return cutscene;
    }
}
